@@ -9,6 +9,9 @@ from .common import _Future, MAX_TIMEOUT, copy_future_exception
 from .wrap import CanCustomizeBind
 from .helpers import executor_loop
 from .event import get_event, is_shutdown
+from .logwrap import LogWrapper
+from .helpers import ShutdownHelper
+from .metrics import metrics, track_future
 
 
 class RetryPolicy(object):
@@ -83,7 +86,7 @@ class ExceptionRetryPolicy(RetryPolicy):
         return False
 
     def sleep_time(self, attempt, future):
-        return min(self._sleep * (self._exponent ** attempt), self._max_sleep)
+        return min(self._sleep * (self._exponent**attempt), self._max_sleep)
 
 
 class RetryJob(object):
@@ -187,7 +190,9 @@ class RetryExecutor(CanCustomizeBind, Executor):
     - This executor is thread-safe.
     """
 
-    def __init__(self, delegate, retry_policy=None, logger=None, **kwargs):
+    def __init__(
+        self, delegate, retry_policy=None, logger=None, name="default", **kwargs
+    ):
         """
         Parameters:
 
@@ -201,34 +206,45 @@ class RetryExecutor(CanCustomizeBind, Executor):
 
             logger (~logging.Logger):
                 a logger used for messages from this executor
+
+            name (str):
+                a name for this executor
+
+        .. versionchanged:: 2.7.0
+            Introduced ``name``.
         """
-        self._log = logger if logger else logging.getLogger("RetryExecutor")
+        self._log = LogWrapper(logger if logger else logging.getLogger("RetryExecutor"))
         self._delegate = delegate
         self._default_retry_policy = retry_policy or ExceptionRetryPolicy(**kwargs)
         self._jobs = []
         self._submit_event = get_event()
+        self._name = name
 
         event = self._submit_event
         self_ref = weakref.ref(self, lambda _: event.set())
         self._submit_thread = Thread(
-            name="RetryExecutor", target=_submit_loop, args=(self_ref,)
+            name="RetryExecutor-%s" % name, target=_submit_loop, args=(self_ref,)
         )
         self._submit_thread.daemon = True
 
-        self._shutdown = False
+        self._shutdown = ShutdownHelper()
         self._lock = RLock()
 
+        metrics.EXEC_INPROGRESS.labels(executor=self._name, type="retry").inc()
+        metrics.EXEC_TOTAL.labels(executor=self._name, type="retry").inc()
+
         self._submit_thread.start()
 
-    def shutdown(self, wait=True):
-        self._log.debug("Shutting down.")
-        self._shutdown = True
-        self._wake_thread()
-        self._delegate.shutdown(wait)
-        if wait:
-            self._log.debug("Waiting for thread")
-            self._submit_thread.join(MAX_TIMEOUT)
-        self._log.debug("Shutdown complete")
+    def shutdown(self, wait=True, **_kwargs):
+        if self._shutdown():
+            self._log.debug("Shutting down.")
+            metrics.EXEC_INPROGRESS.labels(executor=self._name, type="retry").dec()
+            self._wake_thread()
+            self._delegate.shutdown(wait, **_kwargs)
+            if wait:
+                self._log.debug("Waiting for thread")
+                self._submit_thread.join(MAX_TIMEOUT)
+            self._log.debug("Shutdown complete")
 
     def submit(self, *args, **kwargs):  # pylint: disable=arguments-differ
         return self.submit_retry(self._default_retry_policy, *args, **kwargs)
@@ -239,16 +255,18 @@ class RetryExecutor(CanCustomizeBind, Executor):
         Parameters:
             retry_policy (RetryPolicy): a policy which is used for this call only
         """
-        future = RetryFuture(self)
+        with self._shutdown.ensure_alive():
+            future = RetryFuture(self)
+            track_future(future, type="retry", executor=self._name)
 
-        job = RetryJob(retry_policy, None, future, 0, monotonic(), fn, args, kwargs)
-        self._append_job(job)
+            job = RetryJob(retry_policy, None, future, 0, monotonic(), fn, args, kwargs)
+            self._append_job(job)
 
-        # Let the submit thread know it should wake up to check for new jobs
-        self._wake_thread()
+            # Let the submit thread know it should wake up to check for new jobs
+            self._wake_thread()
 
-        self._log.debug("Returning future %s", future)
-        return future
+            self._log.debug("Returning future %s", future)
+            return future
 
     def _wake_thread(self):
         self._submit_event.set()
@@ -292,6 +310,9 @@ class RetryExecutor(CanCustomizeBind, Executor):
                     )
                     return
 
+                if job.attempt != 0:
+                    metrics.RETRY_TOTAL.labels(executor=self._name).inc()
+
                 delegate_future = self._delegate.submit(job.fn, *job.args, **job.kwargs)
                 job.future.delegate_future = delegate_future
 
@@ -315,10 +336,12 @@ class RetryExecutor(CanCustomizeBind, Executor):
         with self._lock:
             for idx, pending in enumerate(self._jobs):
                 if pending is job:
+                    metrics.RETRY_QUEUE.labels(executor=self._name).dec()
                     return self._jobs.pop(idx)
 
     def _append_job(self, job):
         with self._lock:
+            metrics.RETRY_QUEUE.labels(executor=self._name).inc()
             self._jobs.append(job)
 
     def _retry(self, job, sleep_time):
@@ -326,6 +349,7 @@ class RetryExecutor(CanCustomizeBind, Executor):
 
         with self._lock:
             self._pop_job(job)
+            metrics.RETRY_DELAY.labels(executor=self._name).inc(sleep_time)
             new_job = RetryJob(
                 job.policy,
                 None,
@@ -469,7 +493,7 @@ def _submit_loop(executor_ref):
         if not executor:
             break
 
-        if executor._shutdown or is_shutdown():
+        if executor._shutdown.is_shutdown or is_shutdown():
             break
 
         executor._log.debug("_submit_loop iter")
